import json
import re
from json import JSONDecodeError

import scrapy

from souhcars.items import BrandItem, SeriesItem, StyleItem
from scrapy.spidermiddlewares.httperror import HttpError
from twisted.internet.error import DNSLookupError, TCPTimedOutError


class SouhCarsSpider(scrapy.Spider):
    name = 'souh_cars'
    allowed_domains = ['auto.sohu.com']
    start_urls = ['http://db.auto.sohu.com/home/']

    def parse(self, response, **kwargs):
        # flag = 0
        all_brand_li = response.xpath('//*[@id="treeNav"]/ul/li[@class="close_child"]')
        for i in all_brand_li:
            # 保存品牌
            brand = BrandItem()
            brand['brand_name'] = i.xpath('h4/a/text()').extract()[1]
            brand['brand_id'] = i.xpath('h4/a').attrib.get('id')
            yield brand

            # 保存车系
            all_maker_li = i.xpath("ul")
            series = SeriesItem()
            series['brand_id'] = i.xpath('h4/a').attrib.get('id')
            for j in all_maker_li:
                series['maker_id'] = j.xpath('li[@class="con_tit"]/a').attrib.get('id')
                series['maker_name'] = j.xpath('li[@class="con_tit"]/a/text()').extract()[1].strip()
                all_series_li = j.xpath('li/a[@class="model-a"]')
                for k in all_series_li:
                    try:
                        series['series_name'] = k.xpath('text()').extract()[1].strip()
                    except IndexError as e:
                        print(e)
                        return
                    series['series_id'] = k.attrib.get('id')
                    yield series
                    # 车的参数配置信息地址
                    style_url ='http:'+ k.attrib.get('href') +'/trim.html'
                    yield scrapy.Request(url=style_url, callback=self.parse_years, errback=self.parse_err)

    def parse_years(self, response, **kwargs):
        all_year = response.xpath('//*[@id="yearp"]/a[@target="_blank"]/text()')
        # 如果只有一个年款，那么上面方式查不到年款，如果查不到，就是用下面方式再查一次
        if len(all_year) == 0:
            # print(response.url)
            all_year = response.xpath('//*[@id="yearp"]/a/text()')
        for i in all_year:
            style_url2 = response.url.replace('trim.html', 'trim_year{}.html'.format(i.root[0:4]))
            yield scrapy.Request(url=style_url2, callback=self.parse_style, errback=self.parse_err)

    def parse_style(self, response, **kwargs):
        regex = re.compile(r"var trimParam = ([\s\S]*?)}];")
        try:
            all_style = json.loads(response.selector.re(regex)[0] + "}]")
        except JSONDecodeError as e:
            print(e)
            print(response.selector.re(regex)[0])
            print(response.url)
            return
        for i in all_style:
            style = StyleItem()
            style['style_id'] = i.get('SIP_T_ID')
            style['style_name'] = i.get('subbrandName') + ' ' + i.get('modelName') + ' ' + str(i.get('SIP_T_YEAR'))\
                                  + '款 ' + i.get('SIP_T_NAME')
            style['series_id'] = i.get('SIP_T_MODELID')
            style['conf'] = str(i).replace("\"", "\\\"")
            yield style

    def parse_err(self, failure):
        if failure.check(HttpError):
            response = failure.value.response
            print('HttpError on {0}'.format(response.url))

        elif failure.check(DNSLookupError):
            request = failure.request
            print('DNSLookupError on {0}'.format(request.url))

        elif failure.check(TimeoutError, TCPTimedOutError):
            request = failure.request
            print('TimeoutError on {0}'.format(request.url))